/*
	File:		EI_MovieImport.c
	
	Description: QuickTime movie import component for Electric Image files

	Author:		??

	Copyright: 	 Copyright 1999 Apple Computer, Inc. All rights reserved.
	
	Disclaimer:	IMPORTANT:  This Apple software is supplied to you by Apple Computer, Inc.
				("Apple") in consideration of your agreement to the following terms, and your
				use, installation, modification or redistribution of this Apple software
				constitutes acceptance of these terms.  If you do not agree with these terms,
				please do not use, install, modify or redistribute this Apple software.

				In consideration of your agreement to abide by the following terms, and subject
				to these terms, Apple grants you a personal, non-exclusive license, under Apples
				copyrights in this original Apple software (the "Apple Software"), to use,
				reproduce, modify and redistribute the Apple Software, with or without
				modifications, in source and/or binary forms; provided that if you redistribute
				the Apple Software in its entirety and without modifications, you must retain
				this notice and the following text and disclaimers in all such redistributions of
				the Apple Software.  Neither the name, trademarks, service marks or logos of
				Apple Computer, Inc. may be used to endorse or promote products derived from the
				Apple Software without specific prior written permission from Apple.  Except as
				expressly stated in this notice, no other rights or licenses, express or implied,
				are granted by Apple herein, including but not limited to any patent rights that
				may be infringed by your derivative works or by other works in which the Apple
				Software may be incorporated.

				The Apple Software is provided by Apple on an "AS IS" basis.  APPLE MAKES NO
				WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
				WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
				PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND OPERATION ALONE OR IN
				COMBINATION WITH YOUR PRODUCTS.

				IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR
				CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
				GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
				ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION
				OF THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT
				(INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN
				ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
				
	Change History (most recent first):

	   <1>	 	11/28/99	???		first file

*/

#define FUNCTION_PASCAL 1
#include <ConditionalMacros.h>
#include <Endian.h>
#include <Movies.h>
#include <QuickTimeComponents.h>

#include "EI_Image.h"
#include "EI_MakeImageDescription.h"
#include "EI_MovieImportVersion.h"

typedef struct {
	ComponentInstance		self;

	long					dataHOffset;
	long					fileSize;
	ComponentInstance		dataHandler;
} EI_MovieImportGlobalsRecord, *EI_MovieImportGlobals;

#define MOVIEIMPORT_BASENAME() 		EI_MovieImport
#define MOVIEIMPORT_GLOBALS() 		EI_MovieImportGlobals storage

#define CALLCOMPONENT_BASENAME()	MOVIEIMPORT_BASENAME()
#define	CALLCOMPONENT_GLOBALS()		MOVIEIMPORT_GLOBALS()

//#define COMPONENT_UPP_PREFIX()		uppMovieImport
#define COMPONENT_DISPATCH_FILE		"EI_MovieImportDispatch.h"
//#define COMPONENT_SELECT_PREFIX()  	kMovieImport
#define COMPONENT_UPP_SELECT_ROOT()		MovieImport

#include "Components.k.h"
#include "QuickTimeComponents.k.h"
#include "ComponentDispatchHelper.c"

#define kTimeScale (600)

pascal ComponentResult EI_MovieImportVersion( EI_MovieImportGlobals store )
{
#pragma unused(store)
	return kEI_MovieImportVersion;
}

pascal ComponentResult EI_MovieImportOpen(EI_MovieImportGlobals store, ComponentInstance self)
{
	OSErr err;

	store = (EI_MovieImportGlobals)NewPtrClear(sizeof(EI_MovieImportGlobalsRecord));
	if (err = MemError()) goto bail;

	store->self = self;

	SetComponentInstanceStorage(self, (Handle)store);

bail:
	return err;
}

pascal ComponentResult EI_MovieImportClose(EI_MovieImportGlobals store, ComponentInstance self)
{
#pragma unused(self)
	if (store)
		DisposePtr((Ptr)store);

	return noErr;
}

pascal ComponentResult EI_MovieImportFile(EI_MovieImportGlobals store, const FSSpec *theFile,
		Movie theMovie, Track targetTrack, Track *usedTrack, TimeValue atTime, TimeValue *durationAdded,
		long inFlags, long *outFlags)
{
	OSErr err = noErr;
	AliasHandle alias = nil;

	*outFlags = 0;

	err = NewAliasMinimal(theFile, &alias);
	if (err) goto bail;

	err = MovieImportDataRef(store->self, (Handle)alias, rAliasType, theMovie, targetTrack,
			usedTrack, atTime, durationAdded, inFlags, outFlags);

bail:
	DisposeHandle((Handle)alias);

	return err;
}

pascal ComponentResult EI_MovieImportDataRef(EI_MovieImportGlobals store, Handle dataRef, OSType dataRefType,
		Movie theMovie, Track targetTrack, Track *usedTrack, TimeValue atTime, TimeValue *durationAdded,
		long inFlags, long *outFlags)
{
	OSErr err = noErr;
	Track videoTrack = nil;
	Media videoMedia;
	ImageDescriptionHandle videoDesc = nil;
	ComponentInstance dataHandler = 0;
	ImageHeader header;
	long fileOffset = 0, fileSize;
	long frameCount;
	Ptr colors = nil;
	TimeValue lastFrameDuration = (kTimeScale / 30);

	*outFlags = 0;

	if (inFlags & movieImportMustUseTrack)
		return paramErr;

	// get a data handler
	err = OpenAComponent(GetDataHandler(dataRef, dataRefType, kDataHCanRead), &dataHandler);
	if (err) goto bail;

	err = DataHSetDataRef(dataHandler, dataRef);
	if (err) goto bail;

	err = DataHOpenForRead(dataHandler);
	if (err) goto bail;

	err = DataHGetFileSize(dataHandler, &fileSize);
	if (err) goto bail;

	// get the file header
	err = DataHScheduleData(dataHandler, (Ptr)&header, fileOffset, sizeof(header), 0, nil, nil);
	if (err) goto bail;

	fileOffset += sizeof(header);

	frameCount = EndianU32_BtoN(header.imageFrames);
	if ((EndianU16_BtoN(header.imageVersion) != 5) || (frameCount < 1)) {
		err = codecBadDataErr;
		goto bail;
	}

	while (frameCount--) {
		ImageFrame frame;
		short colorCount;
		QTFloatSingle frameTimeFloat;
		TimeValue frameTime;
		TimeValue frameDuration;
		long frameSize;

		err = DataHScheduleData(dataHandler, (Ptr)&frame, fileOffset, sizeof(frame), 0, nil, nil);
		if (err) goto bail;

		colorCount = EndianU16_BtoN(frame.framePalettes);

		colors = NewPtr(3 * colorCount);
		if (err = MemError()) goto bail;

		err = DataHScheduleData(dataHandler, colors, fileOffset + sizeof(frame), 3 * colorCount, 0, nil, nil);
		if (err) goto bail;

		err = EI_MakeImageDescription(&frame, colorCount, (UInt8 *)colors, &videoDesc);
		if (err) goto bail;

		if (videoTrack == nil) {
			// create a video track
			videoTrack = NewMovieTrack(theMovie, (**videoDesc).width << 16, (**videoDesc).height << 16, kNoVolume);
			if (err = GetMoviesError()) goto bail;

			videoMedia = NewTrackMedia(videoTrack, VideoMediaType, kTimeScale,
				dataRef, dataRefType);
			if (err = GetMoviesError()) goto bail;

			SetTrackEnabled(videoTrack, true);
		}

		frameSize = EndianU32_BtoN(frame.frameSize) + sizeof(frame) + (3 * colorCount);

		// Endian-flipping floats looks awful, doesn't it?
		*(UInt32 *)&frameTimeFloat = EndianU32_BtoN( *(UInt32 *)&frame.frameTime );
		frameTime = (kTimeScale * frameTimeFloat);
		if (frameCount == 0)
			frameDuration = lastFrameDuration;
		else {
			// look ahead for next frame time to determine duration of this frame
			QTFloatSingle nextFrameTimeFloat;
			TimeValue nextFrameTime;

			err = DataHScheduleData(dataHandler, (Ptr)&frame, fileOffset + frameSize, sizeof(frame), 0, nil, nil);
			if (err) goto bail;

			*(UInt32 *)&nextFrameTimeFloat = EndianU32_BtoN( *(UInt32 *)&frame.frameTime );
			nextFrameTime = (kTimeScale * nextFrameTimeFloat);

			frameDuration = nextFrameTime - frameTime;

			lastFrameDuration = frameDuration;
		}

		// add the sample
		err = AddMediaSampleReference(videoMedia, fileOffset,
				frameSize, frameDuration,
				(SampleDescriptionHandle)videoDesc, 1, 0, nil);
		if (err) goto bail;

		// increment to the next frame
		fileOffset += frameSize;

		// dispose of temporary pieces
		DisposeHandle((Handle)videoDesc);
		videoDesc = nil;

		DisposePtr(colors);
		colors = nil;
	}

	// insert the added media into the track
	err = InsertMediaIntoTrack(videoTrack, atTime, 0,
					GetMediaDuration(videoMedia), 0x010000);
	if (err) goto bail;

	// report the duration added
	*durationAdded = GetTrackDuration(videoTrack) - atTime;

bail:
	if (videoTrack) {
		if (err) {
			DisposeMovieTrack(videoTrack);
			videoTrack = nil;
		}
		else
			*outFlags |= movieImportCreateTrack;
	}

	if (videoDesc)
		DisposeHandle((Handle)videoDesc);

	if (colors)
		DisposePtr(colors);

	if (dataHandler)
		CloseComponent(dataHandler);

	if (usedTrack) *usedTrack = videoTrack;

	return err;
}

pascal ComponentResult EI_MovieImportGetMIMETypeList(EI_MovieImportGlobals store, QTAtomContainer *retMimeInfo)
{
	// Note that GetComponentResource is only available in QuickTime 3.0 or later.
	// However, it's safe to call it here because GetMIMETypeList is only defined in QuickTime 3.0 or later.
	return GetComponentResource((Component)store->self, FOUR_CHAR_CODE('mime'), 512, (Handle *)retMimeInfo);
}

#ifndef STAND_ALONE

void EI_MovieImporterRegister(void);
void EI_MovieImporterRegister(void)
{
	ComponentDescription foo;
	ComponentRoutineUPP componentEntryPoint = NewComponentRoutineProc(EI_MovieImportComponentDispatch);

  	foo.componentType = MovieImportType;
  	foo.componentSubType = FOUR_CHAR_CODE('EIDI');
  	foo.componentManufacturer = VideoMediaType;
  	foo.componentFlags = canMovieImportFiles | canMovieImportInPlace | canMovieImportDataReferences;
  	foo.componentFlagsMask = 0;

	RegisterComponent(&foo, (void *)componentEntryPoint, 0, 0, 0, 0);
}

#endif
