/*
	File:		EI_GraphicsImport.c
	
	Description: Graphics Importer Component code

	Author:		QuickTime team

	Copyright: 	 Copyright 1999 Apple Computer, Inc. All rights reserved.
	
	Disclaimer:	IMPORTANT:  This Apple software is supplied to you by Apple Computer, Inc.
				("Apple") in consideration of your agreement to the following terms, and your
				use, installation, modification or redistribution of this Apple software
				constitutes acceptance of these terms.  If you do not agree with these terms,
				please do not use, install, modify or redistribute this Apple software.

				In consideration of your agreement to abide by the following terms, and subject
				to these terms, Apple grants you a personal, non-exclusive license, under Apples
				copyrights in this original Apple software (the "Apple Software"), to use,
				reproduce, modify and redistribute the Apple Software, with or without
				modifications, in source and/or binary forms; provided that if you redistribute
				the Apple Software in its entirety and without modifications, you must retain
				this notice and the following text and disclaimers in all such redistributions of
				the Apple Software.  Neither the name, trademarks, service marks or logos of
				Apple Computer, Inc. may be used to endorse or promote products derived from the
				Apple Software without specific prior written permission from Apple.  Except as
				expressly stated in this notice, no other rights or licenses, express or implied,
				are granted by Apple herein, including but not limited to any patent rights that
				may be infringed by your derivative works or by other works in which the Apple
				Software may be incorporated.

				The Apple Software is provided by Apple on an "AS IS" basis.  APPLE MAKES NO
				WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
				WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
				PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND OPERATION ALONE OR IN
				COMBINATION WITH YOUR PRODUCTS.

				IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR
				CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
				GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
				ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION
				OF THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT
				(INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN
				ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
				
	Change History (most recent first):


*/

#include <Endian.h>
#include <ImageCompression.h>
#include <Movies.h>
#include <Script.h>
#include <Strings.h>

#ifndef __Prefix_File__
#include "WinPrefix.h"
#endif

#include "EI_Image.h"
#include "EI_MakeImageDescription.h"

#include "EI_GraphicsImportVersion.h"

typedef struct {
	ComponentInstance		self;
	ComponentInstance		parent;
	ComponentInstance		delegateComponent;
} EI_GraphicsImportGlobalsRecord, *EI_GraphicsImportGlobals;

#define GRAPHICSIMPORT_BASENAME() 		EI_GraphicsImport
#define GRAPHICSIMPORT_GLOBALS() 		EI_GraphicsImportGlobals storage

#define CALLCOMPONENT_BASENAME()		GRAPHICSIMPORT_BASENAME()
#define	CALLCOMPONENT_GLOBALS()			GRAPHICSIMPORT_GLOBALS()

#define COMPONENT_UPP_PREFIX()			uppGraphicsImport
#define COMPONENT_DISPATCH_FILE			"EI_GraphicsImportDispatch.h"
#define COMPONENT_SELECT_PREFIX()  		kGraphicsImport

#define	GET_DELEGATE_COMPONENT()		(storage->delegateComponent)

#include "Components.k.h"
#include "ImageCompression.k.h"
#include "ComponentDispatchHelper.c"

pascal ComponentResult EI_GraphicsImportOpen (EI_GraphicsImportGlobals store, ComponentInstance self)
{
	ComponentResult err = noErr;

	store = (EI_GraphicsImportGlobals)NewPtrClear(sizeof(EI_GraphicsImportGlobalsRecord));
	if (err = MemError()) goto bail;

	SetComponentInstanceStorage(self, (Handle)store);

	store->self = self;
	store->parent = self;

	err = OpenADefaultComponent(GraphicsImporterComponentType, GraphicsImporterComponentType, &store->delegateComponent);
	if (err) goto bail;

	err = ComponentSetTarget(store->delegateComponent, self);
	if (err) goto bail;

bail:
	return err;
}

pascal ComponentResult EI_GraphicsImportClose (EI_GraphicsImportGlobals store, ComponentInstance self)
{
#pragma unused(self)
	if (store) {
		if (store->delegateComponent)
			CloseComponent(store->delegateComponent);

		DisposePtr((Ptr)store);
	}
	return noErr;
}


pascal ComponentResult EI_GraphicsImportVersion (EI_GraphicsImportGlobals store)
{
#pragma unused(store)
#ifndef POWERPC_QT
	return kEI_GraphicsImportVersion;
#else
	return kEI_GraphicsImportVersionPPC;
#endif
}

pascal ComponentResult EI_GraphicsImportTarget (EI_GraphicsImportGlobals store, ComponentInstance target)
{
	store->parent = target;
	return noErr;
}

pascal ComponentResult EI_GraphicsImportGetImageDescription (EI_GraphicsImportGlobals store, ImageDescriptionHandle *descOut)
{
	ComponentResult err = noErr;
	ImageFrame frame;
	unsigned long offset, size;
	short colorCount;
	Ptr colors = nil;

	if (!descOut)
		return paramErr;

	err = GraphicsImportGetDataOffsetAndSize(store->parent, &offset, &size);
	if (err) goto bail;

	err = GraphicsImportReadData(store->parent, &frame, offset, sizeof(frame));
	if (err) goto bail;

	colorCount = EndianU16_BtoN(frame.framePalettes);

	colors = NewPtr(3 * colorCount);
	if (err = MemError()) goto bail;

	err = GraphicsImportReadData(store->parent, colors, offset + sizeof(frame), 3 * colorCount);
	if (err) goto bail;

	err = EI_MakeImageDescription(&frame, colorCount, (UInt8 *)colors, descOut);
	if (err) goto bail;

bail:
	if (colors)
		DisposePtr(colors);

	return err;	
}

pascal ComponentResult EI_GraphicsImportGetDataOffsetAndSize (EI_GraphicsImportGlobals store, unsigned long *offsetOut, unsigned long *sizeOut)
{
	ComponentResult err = noErr;
	ImageHeader header;
	ImageFrame frame;
	unsigned long offset, size;

	err = GraphicsImportGetDataOffsetAndSize(store->delegateComponent, &offset, &size);
	if (err) goto bail;

	err = GraphicsImportReadData(store->parent, &header, offset, sizeof(header));
	if (err) goto bail;

	if ((EndianU16_BtoN(header.imageVersion) != 5) || (EndianU32_BtoN(header.imageFrames) < 1)) {
		err = codecBadDataErr;
		goto bail;
	}

	err = GraphicsImportReadData(store->parent, &frame, offset + sizeof(header), sizeof(frame));
	if (err) goto bail;

	if (offsetOut) *offsetOut = offset + sizeof(ImageHeader);
	if (sizeOut) *sizeOut = sizeof(ImageFrame) + (EndianU16_BtoN(frame.framePalettes) * 3) + EndianU32_BtoN(frame.frameSize);

bail:
	return err;
}

pascal ComponentResult EI_GraphicsImportValidate (EI_GraphicsImportGlobals store, Boolean *valid)
{
	ComponentResult err = noErr;
	unsigned char header[2];

	*valid = false;

	err = GraphicsImportReadData(store->parent, &header[0], 0, 2);
	if (err) goto bail;
	
	// If your image format has a "magic number" at the start, this is where you should look for it.
	
	// Electric Image files don't have a magic number, but they do have a version code, which should
	// be a big-endian two-byte integer "5".  So we'll check that. 
	
	if( ( 0 == header[0] )
	 && ( 5 == header[1] ) )
		*valid = true;

bail:
	return err;
}

pascal ComponentResult EI_GraphicsImportGetMetaData (EI_GraphicsImportGlobals store, void *userData)
{
	ComponentResult err = noErr;
	Handle	dataHandle = NewHandle(0);

	// For the sake of example we'll return a fixed string as metadata.
	char theCString[255] = "File created with Electric Image";
	StringPtr thePString = c2pstr( theCString );

	if (nil == userData)
	{
		err = paramErr;
		goto bail;
	}

	if (nil == dataHandle)
	{
		err = memFullErr;
		goto bail;
	}
	
	err = PtrAndHand(&thePString[1], dataHandle, thePString[0]);
	if (err) goto bail;
	
	err = AddUserDataText(userData, dataHandle, kUserDataTextInformation, 1, langEnglish);
	if (err) goto bail;

bail:
	if (dataHandle)
		DisposeHandle(dataHandle);

	return err;
}

pascal ComponentResult EI_GraphicsImportGetMIMETypeList(EI_GraphicsImportGlobals store, void *qtAtomContainerPtr)
{
	// Note that GetComponentResource is only available in QuickTime 3.0 or later.
	// However, it's safe to call it here because GetMIMETypeList is only defined in QuickTime 3.0 or later.
	return GetComponentResource((Component)store->self, FOUR_CHAR_CODE('mime'), 128, (Handle *)qtAtomContainerPtr);
}

#ifndef STAND_ALONE

extern void EI_GraphicsImporterRegister(void);
void EI_GraphicsImporterRegister(void)
{
	ComponentDescription foo;
	ComponentRoutineUPP componentEntryPoint = NewComponentRoutineProc(EI_GraphicsImportComponentDispatch);

  	foo.componentType = GraphicsImporterComponentType;
  	foo.componentSubType = FOUR_CHAR_CODE('EIDI');
  	foo.componentManufacturer = kAppleManufacturer;
  	foo.componentFlags = 0;
  	foo.componentFlagsMask = 0;

	RegisterComponent(&foo, (void *)componentEntryPoint, 1, 0, 0, 0);
}

#endif
