/*
	File:		PMUtilities.c
	
	Contains:	QuickTime sample code

	Copyright:	 2000 by Apple Computer, Inc. All rights reserved


*/

#include <MacTypes.h>
#include <Gestalt.h>
#include <Movies.h>
#include "ComFramework.h"
#include "MacFramework.h"
#include "PMUtilities.h"

// Global UPP for PrePreroll
// A routine descriptor for our pre-preroll completion routine
MoviePrePrerollCompleteUPP gMoviePPRollCompleteProc = NULL;

//////////
//
// QTIsQuickTimeInstalled
// Is QuickTime installed?
//
//////////

Boolean IsQuickTimeInstalled (void) 
{
	long		myAttrs;
	OSErr 		myErr = noErr;

	myErr = Gestalt(gestaltQuickTime, &myAttrs);

	return(myErr == noErr);
}


//////////
//
// QTQuickTimeCFMInstalled
// Are the QuickTime CFM libraries installed?
//
//////////

#if TARGET_CPU_PPC
Boolean IsQuickTimeCFMInstalled (void) 
{
	Boolean 	myQTCFMAvail = false;
	long		myAttrs;
	OSErr 		myErr = noErr;

	// Test whether the PowerPC QuickTime glue library is present
	myErr = Gestalt(gestaltQuickTimeFeatures, &myAttrs);
	if (myErr == noErr)
		if (myAttrs & (1L << gestaltPPCQuickTimeLibPresent))
			myQTCFMAvail = true;

	// Test whether a function is available (the library is not moved from the Extension folder);
	// This is the trick to be used when testing if a function is available via CFM
	if (!CompressImage)
		myQTCFMAvail = false;     

	return(myQTCFMAvail);
}
#endif

void DoInitQuickTime( void )
{
	OSErr myErr;
	
	myErr = EnterMovies();
	if (myErr != noErr) {
		QTFrame_ShowWarning("\pCould not initialize QuickTime. Exiting.", myErr);
		ExitToShell();
	}
	
	gMoviePPRollCompleteProc = NewMoviePrePrerollCompleteProc(MoviePrePrerollCompleteProc);
	
	return;
}

//////////
//
// GetWindowPositionFromFile
// Return, through thePoint, the stored position of the specified movie.
//
// Return an error if the movie has no stored position. In any case, return a meaningful position.
//
//////////

OSErr GetWindowPositionFromFile (Movie theMovie, Point *thePoint)
{
	UserData		myUserData = NULL;
	Point			myPoint = {kDefaultWindowX, kDefaultWindowY};
	OSErr			myErr = paramErr;
	
	if (theMovie == NULL)
		goto bail;
		
	// get the movie's user data list
	myUserData = GetMovieUserData(theMovie);	// Movie specifier
	
	if (myUserData != NULL) {
		myErr = GetUserDataItem(myUserData,				// User data list specifier
								&myPoint,				// Recive data pointer
								sizeof(Point),			// Size of the item
								FOUR_CHAR_CODE('WLOC'),	// Item type specifier	
								0);						// Index
		if (myErr == noErr) {
			//  Endian Flipping Big to Native - This will do the right thing on Windows
			myPoint.v = EndianS16_BtoN(myPoint.v);
			myPoint.h = EndianS16_BtoN(myPoint.h);
		}
	}

bail:
	*thePoint = myPoint;

	return(myErr);
}

//////////
//
// MoviePrePrerollCompleteProc
// A completion procedure for preprerolling movies.
//
// The theRefCon parameter is assumed to be a window object.
//
//////////

PASCAL_RTN void MoviePrePrerollCompleteProc (Movie theMovie, OSErr thePrerollErr, void *theRefCon)
{
#pragma unused(thePrerollErr)

	WindowObject		myWindowObject = (WindowObject)theRefCon;

	if ((theMovie == NULL) || (myWindowObject == NULL))
		return;
	
	PrerollAndPlay(theMovie);
}

//////////
//
// Preroll
//
// Preroll the movie, so that it's ready to play when we call StartMovie
//
//////////

void PrerollAndPlay( Movie theMovie )
{
	TimeValue			myTimeValue;
	Fixed				myPlayRate;
	
	
	myTimeValue = GetMovieTime(theMovie,	// Movie specifier
							   NULL);		// Pointer to a time struct.
	
	myPlayRate = GetMoviePreferredRate(theMovie); // Movie specifier
	
	PrerollMovie(theMovie,		// Movie specifier
				 myTimeValue,	// Starting time of segment to play as a TimeValue
				 myPlayRate);	// Playback rate
				 
	StartPlayingMovie(theMovie, myPlayRate);
}

//////////
//
// StartPlayingMovie
// If the movie is at the end it will take it back to the beginning then start playing it.
//
//////////

void StartPlayingMovie(Movie theMovie, Fixed thePlayRate)
{

	long theLoopingInfo;
	QTUtils_GetMovieFileLoopingInfo(theMovie, &theLoopingInfo);
	if(theLoopingInfo == kNoLooping)
		if(GetMovieTime(theMovie, NULL) == GetMovieDuration(theMovie) && !QTUtils_IsStreamedMovie(theMovie))
			GoToBeginningOfMovie(theMovie);
			
	SetMovieRate(theMovie, thePlayRate);
}